unit DaemonServiceMain;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, DaemonApp, EpgProcessor, eventlog, DateUtils, LResources, registry, MPlayerDebugerPipeServer;

type


  { XMLEPGServiceThread }

  XMLEPGServiceThread = class(TThread)
  private
    fLastTimeProcess,fDefaultProcessTime:TDateTime;
    fDelayTime: Single;
    EPGProcessor:XMLEPGProcessor;
    FLog : TEventLog;
    fDebugConnected:Boolean;
    DebugPipe : TMplayerPipeClient;
    procedure OnConnectEvent(Sender: TObject; Connected: boolean);
    function downloadepg: boolean;
    procedure UpdateEPG;
    procedure UpdateInfo(Sender:TObject;Data:String);
  Public
    Constructor Create(ALog : TEventLog);
    //destructor Destroy; override;
    Procedure Execute; override;
    procedure SetupDebugThread;
    procedure KillDebugThread;
    procedure ConnectEvent(Sender: TObject; Connected: boolean);
    procedure SendDebugMSG(Data:String);
    procedure WriteDebugLog(LogData: String);


  end;


  { TXMLEPGDaemon }

  TXMLEPGDaemon = class(TDaemon)
    procedure XMLEPGDaemonCreate(Sender: TObject);
    procedure XMLEPGDaemonExecute(Sender: TCustomDaemon);
    procedure XMLEPGDaemonStart(Sender: TCustomDaemon; var OK: Boolean);
    procedure XMLEPGDaemonStop(Sender: TCustomDaemon; var OK: Boolean);
  private
    { private declarations }
    FLog : TEventLog;
    TEPGThread:XMLEPGServiceThread;
    procedure StartLog;
    procedure TerminateEvent(Sender:TObject);
  public
    { public declarations }
  end;

var
  XMLEPGDaemon: TXMLEPGDaemon;

implementation

procedure RegisterDaemon; 
begin
  RegisterDaemonClass(TXMLEPGDaemon)
end;

procedure TXMLEPGDaemon.XMLEPGDaemonStart(Sender: TCustomDaemon; var OK: Boolean);
begin
  Ok := true;
end;

procedure TXMLEPGDaemon.XMLEPGDaemonCreate(Sender: TObject);
begin

end;

procedure TXMLEPGDaemon.XMLEPGDaemonExecute(Sender: TCustomDaemon);
var
  I:integer;
begin
  StartLog;
  TEPGThread:=nil;
  fLog.Log('XMLEPG.Daemon Starting..');
  if Not Assigned(TEPGThread) then
  begin
    fLog.Log('XMLEPGServiceThread Starting..');
    TEPGThread:= XMLEPGServiceThread.Create(flog);
    flog.Log('XMLEPGServiceThread Created');
    if TEPGThread.Suspended then
    begin
      fLog.Log('XMLEPGServiceThread Resumimg..');
      TEPGThread.Resume;
    end
    else
      flog.Log('XMLEPGServiceThread Active...');

  end
  else
  begin
    fLog.Log('Cant Start XMLEPGServiceThread ');
    TEPGThread:=nil;
  end;

(*  flog.Log('XMLEPGDaemon.Execute');
  if TEPGThread <> nil then
  begin
    TEPGThread.fDebugConnected := false;
    TEPGThread.DebugConnect;
    TEPGThread.Resume;
    flog.Log('TEPGThread Running');
  end
  else
  begin
    fLog.Log('XMLEPGServiceThread Starting2..');
    TEPGThread:= XMLEPGServiceThread.Create(flog,@TerminateEvent);
    TEPGThread.fDebugConnected := false;
    TEPGThread.DebugConnect;
    TEPGThread.Resume;
    flog.Log('TEPGThread Running');
  end;*)
end;


procedure TXMLEPGDaemon.XMLEPGDaemonStop(Sender: TCustomDaemon; var OK: Boolean);
begin
  fLog.Log('XMLEPG Daemon Stopping..');
  if Assigned(TEPGThread) then
    TEPGThread.free;
  fLog.Log('XMLEPG Daemon Stoped..');
  ok :=true;

end;

procedure TXMLEPGDaemon.StartLog;
begin
  fLog := self.Logger;
  flog.Log('XMLEPG LOGGING Started');
end;

procedure TXMLEPGDaemon.TerminateEvent(Sender: TObject);
begin
  fLog.Log('XMLEPG Daemon Exiting..');
  TEPGThread := nil;
end;



{ TXMLEPGDaemon }


{ XMLEPGServiceThread }

procedure XMLEPGServiceThread.OnConnectEvent(Sender: TObject; Connected: boolean);
begin

end;

function XMLEPGServiceThread.DownloadEPG: Boolean;
var
  A0,A1,A2,A3:Single;
  S0,S1,S2,S3:String;
begin
  A0 := HoursBetween(Now,fDefaultProcessTime);
  A1 := HoursBetween(now,fLastTimeProcess);
  A2 := MinutesBetween(Now,fDefaultProcessTime);
  A3 := MinutesBetween(now,fLastTimeProcess);

  S0 := 'Hours Between Now & Default Process Time '+FloatToStrf(A0,fffixed,7,1);
  S1 := 'Hours Between Now & Last Process Time '+FloatToStrf(A1,fffixed,7,1);
  S2 := 'Minutes Between Now & Default Process Time '+FloatToStrf(A2,ffFixed,7,1);
  S3 := 'Minutes Between Now & Last Process Time '+FloatToStrf(A3,fffixed,7,1);
  SendDebugMSG(S0+#13#10+S1+#13#10+S2+#13#10+S3);
  if S0 <> '' then
  begin
    S0:='';

  end;
  if  not FileExists(ExtractFilePath(Application.ExeName)+'Delphians_XML_EPG.xml') then
  begin
    result := true;
    //SendDebugMSG('No XML File');
    exit;
  end;
  result := false;
  if (HoursBetween(now,fLastTimeProcess) >= 24)and(fDelayTime <> 1) then
  begin
    Result := true;
  //  SendDebugMsg('Hours Betwen Now & Last Process Time '+IntToStr(HoursBetween(now,fLastTimeProcess)));
  end
  else
  begin
    if MinutesBetween(Now,fDefaultProcessTime) = 0 then
    begin
      Result :=true;
  //    SendDebugMsg('Mins Between Now & DefaultProcessTime '+IntToStr(MinutesBetween(Now,fDefaultProcessTime)));
    end
    else
    if ((HoursBetween(Now,fDefaultProcessTime) >= 1)and(HoursBetween(now,fLastTimeProcess) >= 1)) and
       ((MinutesBetween(Now,fDefaultProcessTime) >= 60)and(MinutesBetween(now,fLastTimeProcess) >= 60)) then
    begin
      Result :=true;
    //  SendDebugMSG('New File '+S0+#13#10+S1+#13#10+S2+#13#10+S3+#13#10);
    end;
  end;
end;

procedure XMLEPGServiceThread.UpdateEPG;
var
  MyReg:TRegistry;
  MyDir:String;
begin
  MyDir := ExtractFilePath(Application.ExeName);
  EPGProcessor := XMLEPGProcessor.Create(MyDir,@UpdateInfo);
  while not EPGProcessor.Finished do
    Sleep(350);

  EPGProcessor.Destroy;
  EPGProcessor := nil;
  MyReg := TRegistry.Create;
  MyReg.RootKey := HKEY_CURRENT_USER;
  MyReg.OpenKey('\Software\XMLEPGDownloader',true);
  MyReg.WriteDate('LastDownloaded',now);
  fLastTimeProcess := Now;
  MyReg.free;
end;

procedure XMLEPGServiceThread.UpdateInfo(Sender:TObject;Data: String);
begin

end;

constructor XMLEPGServiceThread.Create(ALog: TEventLog);
var
  MyReg: TRegistry;
begin
  fLog := Alog;
  flog.Log('Daemon Starting Processes');
  Inherited Create(false);
end;



procedure XMLEPGServiceThread.Execute;
Var
  i:integer;
  fLastDebug:TDateTime;
begin
  SetupDebugThread;

  SendDebugMSG('XMLEPGServiceThread.Begin Execute');
  while Not Terminated do
  begin
    SendDebugMSG('XMLEPGServiceThread.Execute');
    if DownloadEPG then
    begin
      SendDebugMSG('XMLEPGServiceThread.Update EPG (BEFORE)');
      UpdateEPG;
      SendDebugMSG('XMLEPGServiceThread.Update EPG (AFTER)');
      for i:=0 to 100 do
        Sleep(round(1*fDelayTime));
    end;

    for i:=1 to 60 do
      sleep(round(10*fDelayTime));
  end;
  SendDebugMSG('XMLEPGServiceThread.Execute Finished.');
  KillDebugThread;
end;

procedure XMLEPGServiceThread.SetupDebugThread;
var
  MyReg:TRegistry;
  ServerName,ClientName,PipeName:string;
begin
  fLastTimeProcess := TDate(Now -1000);
  if ExtractFileNameOnly(ExtractFileNameWithoutExt(Application.ExeName)) = 'EpgXMLImporter' then
  begin
    fDelayTime := 1;
    fDefaultProcessTime := EncodeTime(HourOf(Now),00,00,00);
  end
  else
  begin
    fDelayTime := 25;
    fDefaultProcessTime := EncodeTime(03,00,00,00);
  end;
  MyReg := TRegistry.Create;
  MyReg.RootKey := HKEY_CURRENT_USER;
  MyReg.OpenKey('\Software\XMLEPGDownloader',true);
  if MyReg.ValueExists('LastDownloaded') then
    fLastTimeProcess := MyReg.ReadDate('LastDownloaded')
  else
    fLastTimeProcess := TDateTime(Now-1000);
  MyReg.free;
  ServerName := '';
  ClientName := 'XMLEPGService';
  pipename := 'XMLEPG_debuger';
  if not Assigned(DebugPipe) then
    DebugPipe := TMplayerPipeClient.Create(servername,pipename,clientName,@ConnectEvent);
end;

procedure XMLEPGServiceThread.KillDebugThread;
begin
  if Assigned(DebugPipe) then
  begin
    DebugPipe.free;
    DebugPipe := nil;
  end;

end;

procedure XMLEPGServiceThread.ConnectEvent(Sender: TObject; Connected: boolean);
begin
  SendDebugMSG('Connect Event '+BoolToStr(Connected));

end;


procedure XMLEPGServiceThread.SendDebugMSG(Data: String);
begin
  WriteDebugLog(Data);
end;

procedure XMLEPGServiceThread.WriteDebugLog(LogData: String);
var
  tf:file;
  Bytesdone:Smallint;
  MyData:PChar;
  FSize:Int64;
  fs,sPath,sLogFile,MyDate:String;
begin
  FS:='DD/MM/YYYY HH:MM:SS:MS';
  MyDate := FormatDateTime(FS,Now);
  sLogFile := 'debuger.log';
  AssignFile(TF,sLogFile);
  if FileExists(sLogFile) then
    Reset(TF,1)
  else
    ReWrite(TF,1);
  Seek(TF,FileSize(sLogFile));
  MyDate := MyDate+' : '+logdata+#13#10;
  MyData := PChar(myDate);
  GetMem(MyData,Length(MyDate)+1);
  MyData:= StrPCopy(MyData,MyDate);
  BlockWrite(TF,MyData^,Length(myDate)+1,Bytesdone);
  FreeMem(MyData,length(MyDate)+1);
  Closefile(TF);
end;

{ TXMLEPGDaemonMapper }


{$R *.lfm}


initialization
  RegisterDaemon;
end.

